;;   Copyright 1989 O'Reilly and Associates, Inc.

;;      The X Consortium, and any party obtaining a copy of these files from
;;      the X Consortium, directly or indirectly, is granted, free of charge, a
;;      full and unrestricted irrevocable, world-wide, paid up, royalty-free,
;;      nonexclusive right and license to deal in this software and
;;      documentation files (the "Software"), including without limitation the
;;      rights to use, copy, modify, merge, publish, distribute, sublicense,
;;      and/or sell copies of the Software, and to permit persons who receive
;;      copies from any such party to do so.  This license includes without
;;      limitation a license to do the foregoing actions under any patents of
;;      the party supplying this software to the X Consortium.

;; #include <X11/Xlib.h>
;; #include <X11/Xutil.h>

;; #include <stdio.h>

;; #include "../bitmaps/icon_bitmap"

(load "../misc.scm")

(load "xlib.o1")
(load "X-ffi.o1")
(load "bit-ops.o1")

(define InputHint 0)
(define PPosition 4)
(define PMinSize  (long-<< 1 4))
(define PSize (long-<< 1 3))

(define IconPixmapHint 4)
(define StateHint 2)
(define InputHint 2)

(define KeyPressMask (long-<< 1 0))
(define ButtonPressMask (long-<< 1 2))
(define ExposureMask  (long-<< 1 15 ))
(define StructureNotifyMask (long-<< 1 17))

(define Expose 12)
(define ButtonPress 4)
(define KeyPress 2)
(define ConfigureNotify 22)

(define LineOnOffDash 1)
(define CapRound 2)
(define JoinRound 1)

(define icon-bitmap-width 4)
(define icon-bitmap-height 40)
(define icon-bitmap-bits
  (list->string
   (map integer->char
	`( #xc3 #xc3 #x7f #x00 #x78 #x00 #x00 #x00 #x00 #xc0 #x00 #x00
		#x00 #x00 #x80 #x38 #x00 #x40 #x00 #x80 #x24 #x00 #x00 #x00
		#x80 #x44 #x00 #x00 #x00 #x80 #x44 #x00 #x00 #x00 #x80 #x74
		#x00 #x0f #x0c #x00 #x7c #x3e #x41 #x0e #x00 #x44 #x22 #x41
		#x02 #x00 #x84 #x22 #x46 #x02 #x00 #x9c #x26 #xcc #x02 #x00
		#x78 #x3c #xcd #x36 #x80 #x00 #x20 #x06 #x0c #x80 #x01 #x00
		#x00 #x00 #x80 #x01 #x02 #x40 #x00 #x80 #x01 #x06 #x40 #x00
		#x80 #x01 #x04 #x20 #x00 #x80 #x01 #x04 #x20 #x01 #x80 #x01
		#x04 #x20 #x00 #x80 #x01 #x04 #x22 #x00 #x80 #x01 #x04 #x33
		#xf1 #x81 #x01 #x88 #x12 #x31 #x03 #x01 #x88 #x12 #x11 #x02
		#x00 #x88 #x12 #x11 #x02 #x00 #x48 #x1a #x11 #x02 #x00 #x70
		#x04 #x19 #x82 #x01 #x00 #x00 #x00 #x80 #x01 #x00 #x00 #x38
		#x80 #x01 #x00 #x00 #xce #x80 #x01 #x00 #x00 #x83 #x81 #x81
		#x07 #x80 #x01 #x81 #xe1 #x04 #xc0 #x00 #x83 #x31 #x08 #x40
		#x00 #x82 #x10 #x08 #x20 #x00 #x82 #x19 #x10 #x30 #x00 #x86
		#x0c #x30 #x18 #x00 #x84 #x04 #x60 #x0e #x00 #xdc #x02 #x80
		#x03 #x00 #x70 #x00 #x00 #x00 #x00 #x00))))

(define BITMAPDEPTH 1)
(define TOO_SMALL 0)
(define BIG_ENOUGH 1)

; /* These are used as arguments to nearly every Xlib routine, so it saves 
;  * routine arguments to declare them global.  If there were 
;  * additional source files, they would be declared extern there. 
; Display *display;
; int screen_num;

; static char *progname; /* name this program was invoked by 
(define progname "basicwin")

;; check the result of evaluating PROC.  If PROC is defined then it
;; decides of the result is OK.  If PROC is not defined, then the
;; result is checked to see if it is not #f.
;;
;; If the result doesn't pass the test, then the program exits,
;; otherwise the result is returned.

(define check-result
  (lambda (proc . pred)
    (let ((result (proc))
	  (pred (if (null? pred) 
		    (lambda (arg) arg)
		    pred)))
      
	  (if (not (pred result))
	      (begin
		(display-line "ERROR executing " proc 
			      " - invalid result value.\n")
		(exit 1))
	      result))))

(define hex
  (lambda (num)
    (number->string num 16)))

(define main
  (lambda ()
    (let* ((display-name  #f)
	   (window-size BIG_ENOUGH)
	   (icon-name "basicwin")
	   (window-name "Basic Window Program")
	   (border-width 4)		; in pixels
	   (size-hints (check-result %XAllocSizeHints))
	   (wm-hints  (check-result %XAllocWMHints))
	   (class-hints (check-result %XAllocClassHint))
	   (windowname (%make-XTextProperty))
	   (iconName (%make-XTextProperty))
	   (count (%make-int))
	   (size-list (%make-XIconSize))
	   (report (%make-XEvent))
	   ;; connect to X server
	   (x-display (check-result (lambda () (%XOpenDisplay display-name))))
	   
	   ;; get screen size from display structure macro
	   (screen-num  (%XDefaultScreen x-display))
	   (display-width  (%XDisplayWidth x-display screen-num))
	   (display-height (%XDisplayHeight x-display screen-num))
	   ;; Note that in a real application, x and y would
	   ;; default to 0 but would be settable from the command
	   ;; line or resource database.
	   (x 0)
	   (y 0)
	   
	   ;; size window with enough room for text
	   (width  (quotient display-width 3))
	   (height (quotient display-height 4))

	   ;;create opaque window
	   (win (%XCreateSimpleWindow 
		 x-display
		 (%XRootWindow x-display screen-num) 
		 x y width height
		 border-width
		 ;;		 (BlackPixel x-display screen-num)
		 (BlackPixel x-display screen-num)
		 ;;		 (WhitePixel x-display screen-num)
		 (WhitePixel x-display screen-num))))
      
      (display-line "BlackPixel = " (BlackPixel x-display screen-num) "\n")
      (display-line "WhitePixel = " (WhitePixel x-display screen-num) "\n")
      (display-line "KeyPressMask = " (hex KeyPressMask) "\n")
      (display-line "ButtonPressMask = " (hex ButtonPressMask) "\n")
      (display-line "ExposureMask = " (hex ExposureMask) "\n")
      (display-line "StructureNotifyMask = " (hex StructureNotifyMask) "\n")
      (display-line "ExposureMask | KeyPressMask= " 
		    (hex (long-bit-or ExposureMask 
				      KeyPressMask
				      ButtonPressMask
				      StructureNotifyMask)) "\n")

      ;; Get available icon sizes from Window manager

      (if (= (%XGetIconSizes x-display
			     (%XRootWindow x-display screen-num)
			     size-list
			     count)
	     0)
	  (display "Window manager didn't set icon sizes - using default.\n"))
      ;; A real application would search through
      ;; size-list here to find an acceptable icon size,
      ;; and then create a pixmap of that size.  This
      ;; requires that the application have data for
      ;; several sizes of icons.
      (let ((icon-pixmap (%XCreateBitmapFromData
			  x-display
			  win
			  icon-bitmap-bits
			  icon-bitmap-width
			  icon-bitmap-height)))

	;; Set size hints for window manager.  The window manager may
	;; override these settings.  Note that in a real application
	;; if size or position were set by the user the flags would be
	;; UPosition and USize, and these would override the window
	;; manager's preferences for this window.

	;; x, y, width, and height hints are now taken from the actual
	;; settings of the window when mapped. Note that PPosition and
	;; PSize must be specified anyway.
	
	(%XSizeHints-flags-set! size-hints
				(long-bit-or PPosition PSize PMinSize))
	(%XSizeHints-min-width-set! size-hints 300)
	(%XSizeHints-min-height-set! size-hints 200)

	;;  These calls store window-name and icon-name into
	;;  XTextProperty structures and set their other fields
	;;  properly.

	(let ((ptr-window-name (%make-char*)))
	  (char*-set! ptr-window-name window-name)
	  (if (= 0 (%XStringListToTextProperty ptr-window-name 1 windowname))
	      (begin
		(display "structure allocation for windowName failed.\n")
		(exit 1))))
	
	(let ((ptr-icon-name (%make-char*)))
	  (char*-set! ptr-icon-name icon-name)
	  (if (= 0 (%XStringListToTextProperty ptr-icon-name 1 iconName))
	      (begin
		(display "structure allocation for iconName failed.\n")
		(exit 1))))

	(%XWMHints-initial-state-set! wm-hints NormalState)
	(%XWMHints-input-set! wm-hints True)
	(%XWMHints-icon-pixmap-set! wm-hints icon-pixmap)
	(%XWMHints-flags-set! 
	 wm-hints 
	 (bit-or StateHint IconPixmapHint InputHint))

	(%XClassHint-res-name-set! class-hints progname)
	;; could be a problem.  A pointer to a scheme string
	;; might disappear...
	(%XClassHint-res-class-set! class-hints "Basicwin")
	
; 	(%XSetWMProperties
; 	 x-display
; 	 win
; 	 windowname
; 	 iconName
; 	 argv 
; 	 argc
; 	 size-hints
; 	 wm-hints 
; 	 class-hints)

	;; Select event types wanted
 	(%XSelectInput
 	 x-display
 	 win
 	 (long-bit-or ExposureMask KeyPressMask ButtonPressMask 
 		      StructureNotifyMask))
	
	;; create GC for text and drawing
	(let* ((font-info (load-font x-display))
	       (gc (getGC win font-info x-display screen-num)))
	  
	  ;; Display window
	  (display "Displaying window...\n")
	  (%XMapWindow x-display win)

	  (display "Entering even loop...\n")

	  ;; get events, use first to display text and graphics
	  (let loop ()
	    (display "Waiting on next event...\n")
	    (%XNextEvent x-display report)
	    (let ((event (%XEvent-type report)))
	      (display-line "event = " event "\n")
	      (cond
	       ((= Expose event)
		;; unless this is the last contiguous expose,
		;; don't draw the window
		(if (= 0 (%XEvent-xexpose-count report))
		    ;; if window too small to use
		    (if (= window-size TOO_SMALL)
			(TooSmall win gc font-info)
			;; place text in window
			(begin
			  (draw-text 
			   win gc font-info width height x-display screen-num)
			  ;; place graphics in window
			  (draw-graphics win gc width height x-display)))))
	       ((= ConfigureNotify event)
		;; window has been resized, change width
		;; and height to send to draw-text and
		;; draw-graphics in next Expose
		(let ((width (%XEvent-xconfigure-width report))
		      (height (%XEvent-xconfigure-height report)))
		  (if (or (< width 
			     (%XSizeHints-min-width size-hints))
			  (< height 
			     (%XSizeHints-min-height size-hints)))
		      (set! window-size TOO_SMALL)
		      (set! window-size BIG_ENOUGH))))
	       ((or (= ButtonPress event)
		    (= KeyPress event))
		(%XUnloadFont x-display (%XFontStruct-fid font-info))
		(%XFreeGC x-display gc)
		(%XCloseDisplay x-display)
		(exit 1))
	       (else
		;; all events selected by StructureNotifyMask
		;; except ConfigureNotify are thrown away here,
		;; since nothing is done with them
		(display "Unhandled event.\n")))
	      (loop))))))))

(define getGC
  (lambda (win font-info x-display screen-num)
    (let* ((valuemask  0) ; /* ignore XGCvalues and use defaults 
	   (line-width 6)
	   (line-style LineOnOffDash)
	   (cap-style CapRound)
	   (join-style JoinRound)
	   (dash-offset 0)
	   (dash-list (list->string (map integer->char '(12 24))))
	   (list-length  2)
	   (values (%make-XGCValues))

	   ;; Create default Graphics Context 
	   (gc (%XCreateGC x-display win valuemask values)))

      ;; specify font
      (%XSetFont x-display gc (%XFontStruct-fid font-info))

      ;; specify black foreground since default window background is 
      ;; white and default foreground is undefined.
      (%XSetForeground x-display gc (BlackPixel x-display screen-num))

      ;; set line attributes 
      (%XSetLineAttributes x-display gc line-width line-style 
			   cap-style join-style)

      ;; set dashes 
      (%XSetDashes x-display gc dash-offset dash-list list-length)

      ;; return the graphic context
      gc)))

(define load-font
  (lambda (x-display)
    (let* ((fontname "9x15")
	   
	   ;; Load font and get font information structure. 
	   (font-info (check-result (lambda ()
				      (%XLoadQueryFont x-display fontname)))))
      font-info)))


(define draw-text
  (lambda (win gc font-info win-width win-height x-display screen-num)
    (let* ((string1 "Hi! I'm a window, who are you?")
	   (string2 "To terminate program; Press any key")
	   (string3 "or button while in this window.")
	   (string4 "Screen Dimensions:")
	   ;; need length for both XTextWidth and XDrawString 
	   (len1 (string-length string1))
	   (len2 (string-length string2))
	   (len3 (string-length string3))

	   ;; get string widths for centering 
	   (width1 (%XTextWidth font-info string1 len1))
	   (width2 (%XTextWidth font-info string2 len2))
	   (width3 (%XTextWidth font-info string3 len3))
	   
	   (font-height (+ (%XFontStruct-ascent font-info)
			   (%XFontStruct-descent font-info))))

      ;; output text, centered on each line 
      (%XDrawString x-display win gc (quotient (- win-width width1) 2)
		    font-height
		    string1 len1)
      (%XDrawString x-display win gc (quotient (- win-width width2) 2 )
		    (- win-height (* 2 font-height))
		    string2 len2)
      (%XDrawString x-display win gc (quotient (- win-width width3) 2)
		    (- win-height font-height)
		    string3 len3)

      ;; copy numbers into string variables 
      (set! cd-height 
	    (string-append 
	     " Height - "
	     (number->string (%XDisplayHeight x-display screen-num))
	     " pixels"))
      (set! cd-width 
	    (string-append
	     " Width  - "
	     (number->string (%XDisplayWidth x-display screen-num))
	     " pixels"))
      (set! cd-depth 
	    (string-append
	     " Depth  - "
	     (number->string (%XDefaultDepth x-display screen-num))
	     " plane(s)"))
      
      ;; reuse these for same purpose 
      (set! len4 (string-length string4))
      (set! len1 (string-length cd-height))
      (set! len2 (string-length cd-width))
      (set! len3 (string-length cd-depth))

      ;; To center strings vertically, we place the first string so
      ;; that the top of it is two font-heights above the center of
      ;; the window.  Since the baseline of the string is what we
      ;; need to locate for XDrawString, and the baseline is one
      ;; font-info->ascent below the top of the character, the final
      ;; offset of the origin up from the center of the window is
      ;; one font-height + one descent.

      (let ((initial-y-offset (- (quotient win-height 2) 
				 font-height 
				 (%XFontStruct-descent font-info)))
	    (x-offset (quotient win-width 4)))
	(%XDrawString x-display win gc x-offset initial-y-offset 
		      string4 len4)

	(%XDrawString x-display win gc x-offset 
		      (+ initial-y-offset font-height)
		      cd-height
		      len1)
	(%XDrawString x-display win gc x-offset 
		      (+ initial-y-offset  (* 2 font-height))
		      cd-width
		      len2)
	(%XDrawString x-display win gc x-offset 
		      (+ initial-y-offset  (* 3 font-height))
		      cd-depth 
		      len3)))))

(define draw-graphics
  (lambda (win gc window-width window-height x-display)
    (let* ((height (quotient window-height 2))
	   (width (* 3 (quotient window-width 4)))
	   (x (- (quotient window-width 2) (quotient width 2)))  ;; center 
	   (y (- (quotient window-height 2) (quotient height 2))))
      (%XDrawRectangle x-display win gc x y width height))))

(define TooSmall
  (lambda (win gc font-info)
    (let ((string1 "Too Small")
	  (y-offset (+ font-info->ascent 2))
	  (x-offset 2))

	;; output text centered on each line 
      (%XDrawString x-display win gc x-offset y-offset string1 
		    (string-length string1)))))

(main)
