#include <assert.h>
#include <stdio.h>
#include "code.h"

/* A gambit ffigen backend test file of contrived examples. */

/* declare variables with lots of different types. */

unsigned int var_uint;
long var_long;
unsigned long var_ulong;

/* long-long's are ok with lcc, but Gambit's ffi doesn't recognize
   them yet ! */
#if 0
long long var_long_long;
unsigned long long var_ulong_long;
#endif

/* these are not declared in the header and so are not picked
   up. That's OK, if they are going to be global, they need to be in
   the header.  Otherwise they should be declared static here. */

float var_float;
double var_double;
char var_char;
unsigned char var_u_char;
unsigned var_unsigned;
short var_short;
unsigned short var_ushort;

/* Declarations using typeefs. */

int_t var_int;
ptr_int_t var_ptr_int = &var_int;

/* This causes a warning when the ffi generator attempts to create an
   accessor for it, since it would return a struct. */

complex var_complex;

/* Also a problem for the same reasons. */

struct polar var_polar;

/* pointers to structs are OK but introduce memory management problems
   which will be handles when gambit 4.0 is released. */

struct polar *var_ptr_polar;

/* OUCH !! This is a perfect example of why you need gambit-c 4.0
We're allocating pointers and now need to worry about memory
allocation.  4.0 provides automagic mechanisms to handle this case. In
this instance you will have provide an explicit free() - YUCH !*/

/* functions which operate on polar format complex numbers. */

f64
x(struct polar *z)
{
  extern double cos(double);

  return z->mag * cos(z->ang);
}

f64
y(struct polar *z)
{
  extern double sin(double);

  return z->mag * sin(z->ang);
}

/* DANGER DANGER Very unsafe procedure.  For example and test purposes
   ONLY */

void
print_polar(char *s, struct polar *z)
{
  printf(s, z->mag, z->ang);
}


/* functions for rectangular format complex numbers. */

f64
Mag(complex *z)
{
  extern double sqrt(double);

  return sqrt((z->real * z->real) + (z->imag * z->imag));
}

char
enum_char(numbers x)
{
  char ret;

  switch(x) {
    case ONE:
      ret = '1';
      break;
    case TWO:
      ret = '2';
      break;
    case THREE:
      ret = '3';
      break;
    default:
      ret = '0';
      break;
  }
  return ret;
}

#if 0
/* Test of a function which takes a pointer to a function argument. */

int
times2(int x)
{
  return 2 * x;
}

int
func(int (*f)(int), int y)
{
  return (*f)(y);
}
#endif

/* $Id: code.c,v 1.2 2001/02/07 07:49:12 briand Exp $ */
