;; The initial portion of the back-end.  Simply read in the ffi file
;; and initialize the data structures.

(define functions '())      ; list of function records
(define vars '())           ; list of var records
(define types '())          ; list of type records
(define structs '())        ; list of struct records
(define unions '())         ; list of union records
(define macros '())         ; list of macro records
(define enums '())          ; list of enum records
(define enum-idents '())    ; list of enum-ident records

(define source-file #f)     ; name of the input file itself
(define filenames '())      ; names of all files in the input

(define warnings 0)

(define error 
  (lambda (proc-name str . args)
    (display-line "ERROR: in " proc-name "\n")
    (display-line str " ")
    (apply display-line args)))

;; Process filename.  filenames is the list of filenames which contain
;; data to be analyzed.  This provides a convenient way of filtering.

(define process
  (lambda (filename)
    (set! source-file filename)
    (set! functions '())
    (set! vars '())
    (set! types '())
    (set! structs '())
    (set! unions '())
    (set! macros '())
    (set! enums '())
    (set! enum-idents '())
    (set! filenames '())
    (set! warnings 0)
    (call-with-input-file filename
      (lambda (p)
	(do ((item (read p) (read p)))
	    ((eof-object? item)
	     ;; only print warnings if there are any.
	     (if (> warnings 0)
		 (begin
		   (newline)
		   (display warnings)
		   (display " warnings.")
		   (newline)))
	     #t)
	  (let ((fn (get-filename item)))
	    (if (not (member fn filenames))
		(set! filenames (cons fn filenames)))
	    (case (car item)
	      ((function) (set! functions (cons item functions)))
	      ((var)      (set! vars (cons item vars)))
	      ((type)     (set! types (cons item types)))
	      ((struct)   (set! structs (cons item structs)))
	      ((union)    (set! unions (cons item unions)))
	      ((macro)    (set! macros (cons item macros)))
	      ((enum)     (set! enums (cons item enums)))
	      ((enum-ident) (set! enum-idents (cons item enum-idents)))
	      (else 
	       (error "process : " item)))))))))

(define warn 
  (let ((out (current-output-port)))
    (lambda (msg . rest)
      (set! warnings (+ warnings 1))
      (display ";; WARNING: " out)
      (display msg out)
      (newline out)
      (for-each
       (lambda (x) 
	 (display ";;          " out)
	 (display x out)
	 (newline out))
       rest)
      (newline out))))

;; $Id: init.scm,v 1.1.1.1 2001/01/31 07:33:35 briand Exp $
