
;; Return a list consisting of records which are from one of the
;; filenames contained in FILENAMES.

(define only-records
  (lambda (records . filenames)
    (filter 
     (lambda (record)
       (let ((record-filename (get-filename record)))
	 (any (lambda (filename)
		(string=? record-filename filename))
	      filenames)))
     records)))

;; Only functions which are defined in one of FILENAMES will be
;; referenced.
;; recs are the records to search through
;; filenames is a listo filenames of interest (must be a full path)
;; names specific names of interest
;;
;; Only records which come from one of FILENAMES or whose name is
;; listed in NAMES will be selected.
;;

(define (select-records recs filenames names)
  (let* ((records (apply only-records recs filenames))
	 (ls
	  (filter
	   (lambda (record)
	     (let ((name (get-name record)))
	       (not (prepends__?  name))))
	   (if (null? names)
	       records
	       (filter
		(lambda (record)
		  (member (get-name record) (car names)))
		records)))))
    (for-each referenced! ls)
    ls))

;; $Id: process.scm,v 1.2 2001/02/03 19:09:32 briand Exp $