; Lookup by the 'name' field of whatever it is.

;; KEY is the key to look for

;; ITEMS is the list of items to search through

(define (lookup key items)
  (do ((items items (cdr items)))
      ((or (null? items)
	   (string=? key (get-name (car items))))
       (if (null? items)
	   #f
	   (car items)))))

(define (referenced? x)
  (car (sysinfo x)))

(define (referenced! x)
  (set-car! (sysinfo x) #t))

(define (unreferenced! x)
  (set-car! (sysinfo x) #f))

(define (cache-name r n)
  (let ((i (sysinfo r)))
    (set-car! (cdr i) (cons n (cadr i)))))

(define (cached-names r)
  (cadr (sysinfo r)))

;; Using RECORDS examine each record for type usage.  This is a
;; considerable headache since we have to examine the fields in
;; structures, arguments of functions, etc... and then mark those
;; types which are referenced as being referenced.  Potentially this
;; is all very circular.  It's important to check to see if things are
;; already referenced, as this will prevent inifite loops, which take
;; too long to complete :-)

(define compute-referenced-types
  (lambda (records types)
    (letrec ((type-ref
	      (lambda (type)
		(case (get-type-tag type)
		  ((struct-ref)
		   (let ((struct (lookup (get-ref-tag type) structs))) 
		     (if (not (referenced? struct))
			 (begin
			   (referenced! struct)
			   (rec-ref struct)))))
		  ((union-ref)
		   (let ((union (lookup (get-ref-tag type) unions)))
		     (if (not (referenced? union))
			 (begin
			   (referenced! union)
			   (rec-ref union)))))
		  ((function) 
		   (for-each type-ref (get-function-arglist type))
		   (type-ref (get-function-return-type type)))
		  ((pointer)
		   (type-ref (get-pointer-type type)))
		  ((array)
		   (type-ref (get-array-type type)))
		  ((typedef)
		   (let ((t (lookup (get-ref-tag type) types)))
		     (referenced! t)
		     (type-ref (get-type t))))
		  (else
		   #t))))
	     (rec-ref 
	      (lambda (t-rec)
		(case (record-tag t-rec)
		  ((function) 
		   (let ((type (get-type t-rec)))
		     (for-each type-ref (get-function-arglist type))
		     (type-ref (get-function-return-type type))))
		  ((var type)
		   (type-ref (get-type t-rec)))
		  ((struct union)
		   (for-each
		    (lambda (field)
		      (type-ref (get-field-type field)))
		    (get-struct-fields t-rec)))
		  (else
		   #t)))))
      (for-each 
       (lambda (rec)
	 (if (referenced? rec)
	     (rec-ref rec)))
       records)
      #t)))

;; $Id: reference.scm,v 1.2 2001/02/03 19:09:32 briand Exp $
