; File name utilities.

(define (strip-extension fn)
  (do ((i (- (string-length fn) 1) (- i 1)))
      ((or (< i 0)
	   (char=? (string-ref fn i) #\.)
	   (char=? (string-ref fn i) #\/))
       (if (and (>= i 0) (char=? (string-ref fn i) #\.))
	   (substring fn 0 i)
	   (string-copy fn)))))

(define (strip-path fn)
  (do ((i (- (string-length fn) 1) (- i 1)))
      ((or (< i 0)
	   (char=? (string-ref fn i) #\/))
       (if (and (>= i 0) (char=? (string-ref fn i) #\/))
	   (substring fn (+ i 1) (string-length fn))
	   (string-copy fn)))))

(define (get-path fn)
  (let ((x (strip-path fn)))
    (if (= (string-length fn) (string-length x))
	x
	(substring fn 0 (- (string-length fn) (string-length x))))))

;; Brain-dead string search algorithm which works by brute force.

(define string-contains
  (lambda (text pattern)
    (let ((text-len (string-length text))
	  (pat-len (string-length pattern)))
      ;; silly check
      (let loop ((i 0)
		 (j 0))
	(cond
	 ((>= j pat-len)
	  (list (- i j) i))
	 ((>= i text-len)
	  #f)
	 ((char=? (string-ref pattern j) (string-ref text i))
	  (loop (+ i 1) (+ j 1)))
	 ((> j 0)
	  (loop i 0))
	 (else
	  (loop (+ i 1) 0)))))))


;; Generate a Scheme name from a C name.  In particular :
;;
;; prepend is a string which will be prepended to the name which is
;; generated.
;;
;; .e.g.
;;
;; Subsitute #\- for #\_
;;
;; There will probably be other additions as time goes on.
;;

(define c-name->scheme-name
  (lambda (name . prepend)
    
    (let* ((len (string-length name))
	   (new-name (make-string len)))
      (do ((i 0 (+ i 1)))
	  ((>= i len)
	   (if (and (not (null? prepend))
		    (string? (car prepend)))
	       (string-append (car prepend) new-name)
	       new-name))
	(if (char=? #\_ (string-ref name i))
	    (string-set! new-name i #\-)
	    (string-set! new-name i (string-ref name i)))))))

;; return true of the string starts with a __

(define prepends__?
  (lambda (str)
    (if (> (string-length str) 2)
	(if (string=? "__" (substring str 0 2))
	    #t
	    #f)
	#f)))

;; An identifier follows the usual definition :
;;
;; [a-z][a-z_0-9]+


(define identifier?
  (lambda (str)
    (if (not (char-alphabetic? (string-ref str 0)))
	#f
	(let loop ((i 1))
	  (cond
	   ((>= i (string-length str))
	    #t)
	   ((or (char-alphabetic? (string-ref str i))
		(char-numeric? (string-ref str i))
		(char=? #\_ (string-ref str i)))
	    (loop (+ i 1)))
	   (else
	    #f))))))

;; add explicit double quotes to a string so that it will be displayed
;; as a string.

(define dquote
  (lambda (str)
    (string-append "\"" str "\"")))

;; $Id: string.scm,v 1.1.1.1 2001/01/31 07:33:35 briand Exp $
